<?php
# Copyright(C) 2004-2007 INL
# Written by Eric Leblond <regit@inl.fr>
#            Vincent Deffontaines <gryzor@inl.fr>
#            Jean Gillaux <jean@inl.fr>
#            Damien Boucard <damien.boucard AT inl.fr>
#
# $Id: check.php 16970 2008-11-25 15:50:25Z haypo $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
#  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

require_once('netmask_list.php');

function check_objects_from_desc(&$objects,$used_desc)
{
  #First, build an array with our network adresses
  $addrs = array();
  if (is_array($used_desc->elts))
  {
    foreach ($used_desc->elts as $desc)
    {
      if (($desc->datas['type'] == 'ipv4') and (!preg_match("/^0\.0\.0\.0/",$desc->datas['addr'])))
        array_push($addrs,$desc->datas['addr']);
    }
  }

  $tabs = $objects->list_tab();
  if (!is_array($tabs))
      return;
  foreach ($tabs as $key=>$unused)
  {
      $object = $objects->get_elt($key);
      foreach ($object->elts as $sub_obj)
      {
          if ($sub_obj->datas['type']!='ipv4')
              continue;
          if (preg_match("/^0\.0\.0\.0/",$sub_obj->datas['net']))
              continue;
          // This element must either have a NULL intersection with the net, or be included in it
          $baddr=($sub_obj->datas['net']);
          foreach ($addrs as $addr)
          {
              $aaddr=($addr);
              if (Net_IPv4::intersect($baddr, $aaddr))
              {
                  if (!Net_IPv4::ipInNetwork($baddr, $aaddr))
                  {
                      $objects->elts[$key]->elts[$sub_obj->datas['ID']]->datas['invalid']=1;
                      add_log(sprintf(_('Invalid IPv4 address: "%s".'), $sub_obj->datas['net']));
                  }
              }
          }
      }
  }
}

function check_acls_order_scheme($acls)
{
#Check for unicity of orders for a given (from,to).
#Check all Acls have orders correctly specified
  $used_desc = getSession('used_desc', false);
  $seen=array();
  if (is_array($acls->elts))
  {
    foreach ($acls->elts as $id=>$acl)
    {
      if (isset($acl->descsorts->elts[$used_desc]) and (is_array($acl->descsorts->elts[$used_desc]->elts)))
      {
        foreach ($acl->descsorts->elts[$used_desc]->elts as $idorder=>$tab)
        {
          if (!isset($seen[$tab->from][$tab->to][$tab->order]))
            $seen[$tab->from][$tab->to][$tab->order]=1;
          else
            throw new Exception(sprintf(
                'Duplicate descsort in ACL #%s (desc #%s, from=%s, to=%s, order=%s)',
                $id, $used_desc, $tab->from, $tab->to, $tab->order));
        }
      }
    }
  }
}

function recompule_acl_descsorts($acl_id, &$ruleset, $desc, $descnets)
{
    $used_desc = getSession('used_desc', false);
    $acls = &$ruleset->acls;
    $acl = &$acls->elts[$acl_id];

    // Remplissons un tableau aves les réseaux qui ne contiennent pas la route
    // par défaut ce tableau va nous servir pour forcer ces réseaux sur les
    // sujets purement authentifiants
    $all_nets_but_dftgtw = array();
    foreach ($desc->networks->elts as $this_net)
    {
        if (!isset ($this_net->datas['dftroute']) or ($this_net->datas['dftroute'] != '1'))
        {
            if (isset($this_net->datas['ID']))
            {
                array_push($all_nets_but_dftgtw,$this_net->datas['ID']);
            }else if (isset($this_net->datas['id']))
            {
                array_push($all_nets_but_dftgtw,$this_net->datas['id']);
            }
        }
    }

    // Liste des réseaux sujets et resources concernes par l'ACL $acl
    $from_net = get_nets('from',$acl->from,$ruleset);
    $to_net = get_nets('to',$acl->to,$ruleset);

    // Liste des identifiants desc.xml concernes par ces réseaux
    $from_desc_ids = fill_seen_tab($from_net,$descnets);
    $to_desc_ids = fill_seen_tab($to_net,$descnets);

    if (in_array('all',$from_net))
    {
        $from_desc_ids = $all_nets_but_dftgtw;
    }


    // Reste a comparer le croisement de ces listes avec ceux stockes dans l'ACL
    // elle-même. Commencons par supprimer les marqueurs superflus
    if (isset($acl->descsorts) and (isset($acl->descsorts->elts[$used_desc])) and (is_array($acl->descsorts->elts[$used_desc]->elts)))
    {
        foreach ($acl->descsorts->elts[$used_desc]->elts as $idorder=>$tab)
        {
            if ((!in_array($tab->from,$from_desc_ids)) or (!in_array($tab->to,$to_desc_ids)))
            {
                // Keys not found in the real world, removing
                unset($acl->descsorts->elts[$used_desc]->elts[$idorder]);
            }
        }
    }

    // Et créons les marqueurs manquants

    // Si une ACL n'a aucun réseau en source, on prend tous les réseaux sauf la
    // route par defaut
    if ($from_desc_ids == array ())
    {
        $from_desc_ids = $all_nets_but_dftgtw;
    }

    foreach ($from_desc_ids as $from)
    {
        foreach ($to_desc_ids as $to)
        {
            if (($from==0) and ($to==0))
            {
                // We don't deal with purely local rules [for now]
                continue;
            }
            // If the rule is local, the marker of the local interface is the opposite of the network's marker
            if ($from == 0)
            {
#           $from = 0 - $to;
                $from = -1;
            }
            if ($to == 0)
            {
#           $to = 0 - $from;
                $to = -1;
            }
            $order = $acl->get_order($used_desc,$from,$to);
            if ($order == 0)
            {
                $acl1=&$acl;
                $acl1->set_order($used_desc,$from,$to,$acls->get_max_order($from,$to)+1,$acls->get_max_order_id()+1);
                $acls->del_elt($acl->ID);
                $acls->add_elt($acl1,$acl->ID);
            }
        }
    }
}

function recompute_all_descsorts(&$ruleset, $desc)
{
    $_SESSION['need_acl_recalc']=0;
    $acls = &$ruleset->acls;
    $list = $acls->ordered_list_tab();
    $descnets = gen_descnets($desc);
    foreach ($list as $acl_id=>$value)
    {
        recompule_acl_descsorts($acl_id, $ruleset, $desc, $descnets);
    }
}

function gen_descnets($desc)
{
      $netlist = new NetmaskList();
      foreach ($desc->networks->elts as $id_d=>$this_desc) {

        $netlist->addNetmask($this_desc->datas['addr'], $this_desc);
      }
      $descnets = $netlist->getArray();
      return($descnets);
}


# Constants used by check_objects_completeness()
$mess_error_check_linkid_array = array (
    1 => _("Authentication error: link points to no element in %s."),
    2 => _("Resource error: link points to no element in %s."),
    3 => _("Protocol error: link points to no element in %s."));
$mess_error_check_object_empty_array = array (
    1 => _("Authentication error: the authentication %s is empty."),
    2 => _("Resource error: the resource %s is empty."),
    3 => _("Protocol error: the protocol %s is empty."),
    5 => _("Application error: the application %s is empty."),
    6 => _("Periodicity error: the periodicity %s is empty."));
$context2page = Array(
    // Page name without ".php" extension
    1 => "auths",
    2 => "resources",
    3 => "protocols",
    4 => "l7rules",
    5 => "applications",
    6 => "periodicities",
    7 => "os",
    8 => "durations",
);

function check_objects_completeness($ruleset, $desc, $object, $context)
{
    global $context2page,
        $mess_error_check_object_empty_array, $mess_error_check_linkid_array;

    $page = $context2page[$context].".php";

    $local_addresses = array();
    if ($desc) {
        $use_url = true;
        foreach($desc->addresses->elts as $addr_obj)
        {
            array_push($local_addresses, $addr_obj->datas['addr']);
        }
    } else {
        $use_url = false;
    }

    $problem=0;
    foreach($object->elts as $id=>$obj)
    {
        // doit finir non vide
        $elt_count=0;
        foreach ($obj->elts as $eid=>$elt)
        {
            $elt_count++;
            if ($elt->datas['type']=='nufw')
            {
                if (!isset($elt->datas['group']) or ($elt->datas['group'] == ''))
                {
                    $url = create_check_element_link($page, $obj, $elt, $use_url);
                    log_error(sprintf(_("Authentication error: Undefined nufw group in %s."), $url));
                    $problem++;
                }

            }
            if ($elt->datas['type']=='proto')
            {
                if (!isset($elt->datas['proto']) or ($elt->datas['proto'] == ''))
                {
                    $url = create_check_element_link($page, $obj, $elt, $use_url);
                    log_error(sprintf(_("Protocol error: undefined protocol in %s."), $url));
                    $problem++;
                }
            }
            if ($elt->datas['type']=='link')
            {
                if (!isset($elt->datas['linkid']) or ($elt->datas['linkid'] == ''))
                {
                    $url = create_check_element_link($page, $obj, $elt, $use_url);
                    log_error(sprintf($mess_error_check_linkid_array[$context], $url));
                    $problem++;
                }
            }
        }
        if ($context != 4) # L7 objects may contain no element
        {
            if ($elt_count==0)
            {
                $url = create_check_element_link($page, $obj, null, $use_url);
                log_error(sprintf($mess_error_check_object_empty_array[$context], $url));
                $problem++;
            }
        }
    }
    if ($problem>0)
        return false;
    return true;
}

function check_proto_can_get_auth($protocols,$proto_nb)
{
    $proto=$protocols->get_elt($proto_nb);
    if (!$proto) {
        return false;
    }
    foreach($proto->elts as $subid=>$sub) {
        if ($sub->datas['type'] != 'link') {
            if (!in_array($sub->datas['proto'], Array('tcp', 'udp')))
                return false;
        }else{
            if (!check_proto_can_get_auth($protocols,$sub->datas['linkid']))
                return false;
        }
    }
    return(true);
}

?>
