<?php
# Copyright(C) 2006-2007 INL
# Written by Eric Leblond <regit@inl.fr>
#            Vincent Deffontaines <gryzor@inl.fr>
#            Jean Gillaux <jean@inl.fr>
#
# $Id: display.php 18156 2009-03-17 22:23:01Z gryzor $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
#  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

require_once('bichain.php');
require_once('html.php');

/**
 * display networks chains
 */
function display_chains($ruleset, $used_desc)
{
    $descs = $ruleset->descs;
    $desc = $descs->elts[$used_desc];
    $networks = $desc->elts;
    $s = "<h2>Bichains</h2>";
    foreach($networks as $key=>$network){
        $data = $network->datas;
        $s .= "<li>".$data['name']."</li>";
    }
    return $s;
}


function print_ask_save_changes()
{
    global $header_ask_list_change;

    if (!$header_ask_list_change)
        return;

    $dom_ids = implode(", ", array_map('quote_text', $header_ask_list_change));
    $message = quote_text(_('Order modified, continue without saving?'));

    $script  = "window.onbeforeunload = function(oEvent) { ";
    $script .= "  var changed = ask_list_change(new Array($dom_ids)); ";
    $script .= "  if (changed) return $message;";
    $script .= "};\n";
    html_javascript($script, true);
}

/**
 * javascript code to run on page load
 */
function print_acls_onload($forward_rules=true)
{
    html_javascript("var bichains_t = Array('bi_BICHAIN');");
    $js = "init_acls_dnd(Array('bi_BICHAIN'));";
    if ($forward_rules) {
        $js .= " grey_on_auth();";
    }
    html_javascript($js, true);
    print_ask_save_changes();
}

function display_network_chains($title, $name, $chains, $selected)
{
    print "<select name=\"$name\">";

    $index = 0;
    foreach($chains as $chain){
        $index++;
        $id = "$name$index";
        if ($chain == '*') {
            $text = _('(All)');
            $value = '';
        } else {
            $text = $chain;
            $value = $chain;
        }
        if ($value == $selected || (!$value and !$selected)) {
            $checked = ' selected="selected"';
        } else {
            $checked = '';
        }
        print '<option value="'.$value.'"'.$checked.'>'.$text."</option>\n";
    }
    print "</select>\n";
}

function display_networks($ruleset, $chains, $chain_from, $chain_to)
{
    print '<div class="filterbox">';

    print '<form action="acls.php" method="post">'."\n";

    array_unshift($chains, '*');

    print _("Network").' ';
    display_network_chains(_('Source'), 'chainfrom', $chains, $chain_from);
    print ' =&gt; ';
    display_network_chains(_('Destination'), 'chainto', $chains, $chain_to);

    echo '<input type="submit" value="'._('Filter').'">';
    print "</form>";

    print '<form action="acls.php" method="post">'."\n";

    echo ' '._('Group').' ';
    $groups = $ruleset->groups->list_tab();
    ksort($groups);
    select_array('group', $groups, getHttp('group'), Array('', _('(All)')));

    echo '<input type="submit" value="'._('Filter').'">';
    print "</form>";

    print "</div>\n";
}

class DisplayAcl
{
    function DisplayAcl($ruleset)
    {
        global $title;
        $this->title = $title;
        $this->ruleset = $ruleset;
        $this->gr_high = null;
        $this->current_acl = '';
        $this->chain_from = null;
        $this->chain_to = null;
        if ($title == 'localsin') {
            $this->acls = $this->ruleset->localsin;
        } else if ($title == 'localsout') {
            $this->acls = $this->ruleset->localsout;
        } else {
            $this->acls = $this->ruleset->acls;
        }
        $this->groups = $this->ruleset->groups;
    }

    /**
     * Display acls of one bichain
     */
    function html_acls_bichain($bichain)
    {
        global $used_desc;

        // Get acls in bichain
        $id_tab = array();
        foreach ($this->acls->elts as $acl)
        {
            if (!bichain_touches_acl($bichain, $acl))
                continue;
            $order = $acl->get_order($used_desc, $bichain['from'], $bichain['to']);
            $id_tab[$order] = $acl->ID;
        }

        $label = $bichain['label'];
        $this->html_acls_bichain2($id_tab, $label, $label);
    }

    function filterbox()
    {
        if ($this->title != 'acls')
            return;
        $used_desc = getSession('used_desc', false);
        $chains = get_chains($this->ruleset, $used_desc);
        display_networks($this->ruleset, $chains, $this->chain_from, $this->chain_to);
    }

    function html_acls_bichain2($id_tab, $h2_title, $bichain)
    {
        // display acls by bichain
        $hidden_inputs  = '<input name="dndSort_BICHAIN" value="" type="hidden">';
        $hidden_inputs .= '<input type="hidden" name="bichain" value="'.$bichain.'">';

        $acl_order_form = '<form onsubmit="aclHandle(bichains_t)" action="'.$this->title.'.php" method="post" style="clear:both;">'."\n".
            $hidden_inputs.
            '<input type="submit" class="submit" value="'._('Save order').'" /> '.
            '<input class="inspector" type="button" value="'._('Restore initial order').'" onclick="resetOrder(\'sortcat\')"/>'.
            '<input type="hidden" name="grhigh" value="'.$this->gr_high.'"/>'.
            '<input type="hidden" name="acl" value="'.$this->current_acl.'"/>'.
            '</form>';

        echo "<div id=\"bi_BICHAIN\" class=\"bichain\">\n";

        $this->filterbox();

        echo $acl_order_form;
        echo "<h2>$h2_title</h2>\n";

        // display acls in bichain
        echo '<ul id="sortcat" class="sortcat sortul">'."\n";
        $this->display_acls_dnd($id_tab, $hidden_inputs);
        echo "</ul>\n";

        echo $acl_order_form;
        echo "</div>\n";
    }

    /**
     * Print all acls
     * do not enable javascript
     */
    function html_all_acls(
            $net_from=null, $net_to=null,
            $show_grp=null, $focus=null)
    {
        $used_desc = getSession('used_desc', false);
        $descs = $this->ruleset->descs;
        $desc = $descs->elts[$used_desc];

        $args_net = Array('from' => $net_from, 'to' => $net_to);
        $nets_id = Array('from' => null, 'to' => null);
        $nets_name = Array('from' => null, 'to' => null);

        $title_acls = 'Acls';
        $title = 'acls';
        $singtitle = 'acl';
        if ($focus == 'localsin')
        {
            $title_acls = 'Local input Acls';
            $title = 'localsin';
            $singtitle = 'local';
        }
        if ($focus == 'localsout')
        {
            $title_acls = 'Local output Acls';
            $title = 'localsout';
            $singtitle = 'local';
        }

        $hidden_inputs = "<input type=\"hidden\" name=\"chainfrom\" value=\"$net_from\">\n";
        $hidden_inputs .= "<input type=\"hidden\" name=\"chainto\" value=\"$net_to\">\n";
        // get id of networks for ALL-X or X-ALL display
        foreach (Array('from', 'to') as $direction){
            $net_s = $args_net[$direction];
            if (isset($net_s) and ($net_s != null) and ($net_s!="")){
                $net_obj = $desc->get_net_by_name($net_s);
                if ($net_s == 'LOCAL')
                {
                    $nets_id[$direction]=-1;
                    $nets_name[$direction]='LOCAL';
                }
                if ($net_obj != null){
                    if (isset($net_obj->datas['id']))
                    {
                        $nets_id[$direction] = $net_obj->datas['id'];
                    }else if (isset($net_obj->datas['ID']))
                    {
                        $nets_id[$direction] = $net_obj->datas['ID'];
                    }
                    $nets_name[$direction] = $net_obj->datas['name'];
                }
            }
        }
        if ($net_from != null){
            $title_acls .= " ".$nets_name['from'].'-ALL';
        }
        if ($net_to != null){
            $title_acls .= ' ALL-'.$nets_name['to'];
        }

        if($show_grp != null){
            $grp = $this->groups->get_elt($show_grp);
            $title_acls .= " (".$grp->name.")";
        }

        echo "<div id=\"all_acls\" class=\"bichain\">\n";
        echo _('Reordering the ACLs is possible only when displayed by networks').".";

        $this->filterbox();

        echo "<h2>$title_acls</h2>";
        echo "\n".'<ul id="chain" class="sortcat sortul">'."\n";
        foreach ($this->acls->ordered_list_tab() as $id => $descacl)
        {
            $acl = $this->acls->get_elt_by_id($id);

            // filter acls in case of ALL-X or X-ALL bichains
            if($nets_id['from'] != null and !acl_touches_network($acl, 'from', $nets_id['from'])) {continue;}
            if($nets_id['to'] != null and !acl_touches_network($acl, 'to', $nets_id['to'])) {continue;}
            if($show_grp != null and $acl->group != $show_grp){continue;}

            $this->_display_one_acl($acl, $id, $id, true, $singtitle, $hidden_inputs);
        }
        echo "</ul>\n";
        echo "</div><!-- all_acls -->\n";
    }

    /**
     * Print all local acls
     * do not enable javascript
     */
    function local_all_acls()
    {
        global $used_desc;

        if ($this->title == 'localsin') {
            $label = _('Local input ACLs');
        } else {
            $label = _('Local output ACLs');
        }

        // Get acls in bichain
        $id_tab = array();
        foreach ($this->acls->ordered_list_tab() as $id => $descacl){
            #$acl = $this->acls->get_elt_by_id($id);
            $id_tab[$id] = $id;
        }

        $this->html_acls_bichain2($id_tab, $label, 'bichain');
    }

    /**
     * Display acls in html
     * print html to display the list of acls, with dnd
     *
     * $id_tab: array of acls id, indexed by order
     * $hidden_inputs: inputs fields to be added to forms
     */
    function display_acls_dnd($id_tab, $hidden_inputs)
    {
        ksort($id_tab);
        $used_desc = getSession('used_desc', false);
        foreach ($id_tab as $acl_order=>$id){
            $acl = $this->acls->elts[$id];
            $this->_display_one_acl($acl, $id, $acl_order, false, $GLOBALS['singtitle'], $hidden_inputs);
        }
    }

    /**
     * Return textual representatin of acl
     */
    function html_acl_resume($acl)
    {
        $resources = $this->ruleset->resources;
        $protocols = $this->ruleset->protocols;

        $subject = $resources->get_elt($acl->from);
        $resource = $resources->get_elt($acl->to);
        $protocol = $protocols->get_elt($acl->proto);

        $text = sprintf(_('ACL #%s: '), $acl->ID);
        if ($subject and $resource) {
            $text .= $subject->name." -&gt; ".$resource->name;
        }
        if ($protocol) {
            $text .= ", ".$protocol->name;
        }
        return $text;
    }

    function _display_one_acl($acl, $id, $acl_order, $group_link, $singtitle, $hidden_inputs)
    {
        global $title, $nufw_firewall, $NO_LOG;

        echo "<li id=\"$acl_order\">\n";
        if ($group_link) {
            echo "<form action=\"$title.php\" method=\"post\" class=\"acldetail\">\n";
        } else {
            echo '<form action="'.$this->title.'.php" method="post" class="acldetail" onsubmit="aclHandle(bichains_t)">';
        }

        $css_class = "sort";
        if ($this->gr_high == $acl->group)
        {
            $css_class .= ' highlightelt';
        }

        $acl_title = $this->html_acl_resume($acl);
        echo "<div title=\"$acl_title\" class=\"$css_class\"";
        if ($group_link) {
            echo " style=\"cursor:auto;\" id=\"$id\"";
        }
        echo ">\n";

        $mygrp = $this->groups->elts[$acl->group];
        if ($mygrp->enabled == 1){
            $strong = '<strong>';
            $strongend = '</strong>';
            $span = 'enabled';
        }else{
            $strong = '';
            $strongend = '';
            $span = 'disabled';
        }

        echo "<div class=\"$span\">$strong";

        # Display ACL group
        echo sprintf("[%03s]", $id).' ';
        if ($group_link)
        {
            if ($this->gr_high != $acl->group) {
                echo "<a href=\"$title.php?grhigh=".$acl->group."\">";
            } else {
                echo "<a href=\"$title.php?grhigh=0\">";
            }
        }
        html_img('images/'.$mygrp->ID.'.png', 'group '.$mygrp->ID);
        if ($group_link) {
            echo "</a>\n";
        }
        echo $strongend.' ';

        # Display ACL description
        if ($acl->log != $NO_LOG)
            html_img('images/log.png', _('Log the packet'));
        if ($singtitle == 'acl' and $nufw_firewall) {
            if ($acl->auth)
                html_img('images/user.png', _('ACL using NuFW authentication'));
            if ($acl->periodicity or $acl->duration)
                html_img('images/time.png', _('Time based ACL'));
        }
        echo truncate_string($acl->name, 35)."\n";
        echo "</div>\n";

        html_hidden('acl', $id);
        html_hidden('grhigh', $this->gr_high);
        echo $hidden_inputs;

        echo '<div class=detail>';
        $decision = $acl->decision;
        html_submit(_('Edit'), Array('class' => "input1 $decision"));
        echo '</div>';
        echo "</div>\n";
        echo '</form>';
        echo "</li>\n";
    }
}

/**
 * print list of acls's bichains
 */
function html_acl_get_bichains($bichain_label, $acl, $bichains, $gr_high)
{
    echo '<h3>'._('ACL networks').'</h3>';

    if (!$bichains)
        return;

    echo "<ul class=\"flist\">";
    foreach ($bichains as $id => $data)
    {
        $label = $data['label'];
        if (!bichain_touches_acl($data,$acl))
            continue;

#        $javascript  = "if (ask_list_change(new Array('sortcat'),'"._('Order modified, continue without saving?')."'))";
#        $javascript .= "{ YAHOO.util.Connect.asyncRequest('GET', 'aclajax.php?bichain=$label&amp;amp;grhigh=$gr_high', callback1,'bichain=INTRANET-DMZ');} ";
#        $javascript .= "return false;";
#        $onclick = " onclick=\"$javascript\"";
        $onclick = '';

        echo "<li><a href=\"acls.php?bichain=$label&grhigh=$gr_high\"$onclick>";
        if ($label == $bichain_label)
            echo "<b>$label</b>";
        else
            echo $label;
        echo "</a></li>";
    }
    echo "</ul>";
}

function display_submenu($name,$tabdisp,$tablink,$high,$i, $sep)
{
    print "<dt onclick=\"javascript:montre_global('header', 'header$i');\">";
    print "<a href=\"#\"";
    if ($high)
        echo " class=\"highlight\"";
    print ">".$name."</a>";
    print $sep;
    print "</dt>";
    print "<div class=\"submenu\" id='header$i'>";
    print "<ul>";
    $j=-1;
    foreach ($tabdisp as $disp) {
        $j++;
        print "<li><a href=\"".$tablink[$j].".php\">$disp</a></li>";
    }
    print "</ul>\n</div>";
}

function print_reduced_link($prefix, $id, $name, $key='elt')
{
    print "<a href=\"$prefix?$key=$id\" title=\"$name\">";
}

function yahoo_ask_help($id, $focus=false)
{
    $show = "      YAHOO.nuface.container.contexthelp$id.show();";
    if ($focus) {
        $show .= ' document.forms["js_'.$id.'"].elements["name"].focus();';
    }

    html_javascript(Array(
        "var visible_$id=0;",
        'YAHOO.namespace("nuface.container");',
        "function dispatch_$id() {",
        "   if (visible_$id == 0){",
        $show,
        "   } else {",
        "      YAHOO.nuface.container.contexthelp$id.hide();",
        "   }",
        "   visible_$id = 1 - visible_$id;",
        "}",
        "function init_$id() {",
        "   YAHOO.nuface.container.contexthelp$id = new YAHOO.widget.Panel(\"contexthelp$id\", { width:\"320px\", visible:false, constraintoviewport:true } );",
        "   YAHOO.nuface.container.contexthelp$id.render();",
        "   YAHOO.util.Event.addListener(\"ask-help-$id\", \"click\", dispatch_$id, YAHOO.nuface.container.contexthelp$id, true);",
        "}",
        'YAHOO.util.Event.addListener(window, "load", init_'.$id.');',
    ), true);
}

function contextual_help($id,$text)
{
    yahoo_ask_help($id);

    print "<div id=\"contexthelp$id\" class=\"nufacehelp\">\n";
    print "  <div class=\"hd\">"._("NuFace help")." - $id</div>\n";
    print "  <div class=\"bd\">\n";
    print $text;
    print "  </div>\n";
    print "</div>";
    print "<img src=\"images/help.gif\" id=\"ask-help-$id\" alt=\"help_$id\">\n";
}

function waiting_panel()
{
    print "<div id=\"content\"></div>\n";

    html_include_javascript("js/yahoo/yahoo/yahoo-min.js", true);
    html_include_javascript("js/yahoo/event/event-min.js", true);

    $js = Array(
        "function init() {",
        "    var content = document.getElementById(\"content\");",
        "    content.innerHTML = \"\";",
        "    if (!YAHOO.example.container.wait) {",
        "        // Initialize the temporary Panel to display while waiting for external content to load",
        "        YAHOO.example.container.wait = ",
        "                new YAHOO.widget.Panel(\"wait\",  ",
        "                                                { width: \"240px\", ",
        "                                                  fixedcenter: true, ",
        "                                                  close: false, ",
        "                                                  draggable: false, ",
        "                                                  modal: true,",
        "                                                  visible: false",
        "                                                } ",
        "                                            );",
        '        YAHOO.example.container.wait.setHeader("'._("Loading, please wait...").'");',
        "        YAHOO.example.container.wait.setBody(\"<img src=\\\"images/rel_interstitial_loading.gif\\\" alt=\\\"loading\\\"/>\");",
        "        YAHOO.example.container.wait.render(document.body);",
        "    }",
        "    // Define the callback object for Connection Manager that will set the body of our content area when the content has loaded",
        "    var callback = {",
        "        success : function(o) {",
        "            content.innerHTML = o.responseText;",
        "            content.style.visibility = \"visible\";",
        "            YAHOO.example.container.wait.hide();",
        "        },",
        "        failure : function(o) {",
        "            content.innerHTML = o.responseText;",
        "            content.style.visibility = \"visible\";",
        "            content.innerHTML = \"CONNECTION FAILED!\";",
        "            YAHOO.example.container.wait.hide();",
        "        }",
        "    }",
        "    // Show the Panel",
        "    YAHOO.example.container.wait.show();",
        "    ",
        "    // Connect to our data source and load the data",
        "    var conn = YAHOO.util.Connect.syncRequest(\"GET\", \"nupyf_runs.php\", callback);",
        "}",
        'YAHOO.namespace("example.container");',
        'YAHOO.util.Event.on("panelbutton", "click", init);',
    );
    html_javascript($js, true);
}

?>
