<?php
# Copyright(C) 2004-2007 INL
# Written by Damien Boucard <damien.boucard AT inl.fr>
#
# $Id: netmask_list.php 10248 2007-10-15 17:00:52Z haypo $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
#  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

require_once 'pear/Net/IPv4.php';

/*
 * Parses a string representing an network address (IP with netmask or IP with
 * bitlength) to an array containing separately the IP address and the netask.
 *
 * @author Vincent Deffontaines <gryzor@inl.fr>
 */
function string_to_net($s) {
  $splitted = preg_split("/\//",$s);
  if (count($splitted)>1)
    return ($splitted);
  if ($splitted[0] == "0.0.0.0")
  {
    array_push($splitted,"0.0.0.0");
    return($splitted);
  }
  array_push($splitted,"255.255.255.255");
  return($splitted);
}

/*
 * This class sort by insertion a list of netmask addresses.
 *
 * Note: This sort only guarantees that an included network
 * is before its including network.
 */
class NetmaskList {
  var $head;

  function NetmaskList($array=null) {
    $this->void = null;
    $this->head = $this->void;
    if ($array) {
      foreach ($array as $addr=>$elt) {
        $this->addNetmask($addr, $elt);
      }
    }
  }

  function addNetmask($addr, $data)
  {
    $addrobj = Net_IPv4::parseAddress($addr);
    if (PEAR::isError($addrobj))
        throw new Exception(sprintf(_('Invalid IPv4 address: "%s".'), $addr));

    if ($this->head == $this->void) {
      $this->head = &new NetmaskElement($addr, $data, $this->void, $this->void);
      return;
    }
    $iterator = &$this->head;
    while (1) {
      # Check if $addr is a network included in the current iterated network
      if (Net_IPv4::ipInNetwork($addrobj, $iterator->address)) {
        $element = &new NetmaskElement($addr, $data, $iterator->previous, $iterator);
        $iterator->previous->next = &$element;
        $iterator->previous = &$element;
        if ($iterator === $this->head) {
          $this->head = &$iterator->previous;
        }
        return;
      }
      # End of the list : we insert not-included network
      if ($iterator->next === $this->void) {
        $iterator->next = &new NetmaskElement($addr, $data, $iterator, $this->void);
        return;
      }
      # Iteration
      $iterator = &$iterator->next;
    }
  }

  function getArray() {
    $result = array();
    for ($iterator=$this->head; $iterator!==$this->void; $iterator=$iterator->next) {
      $result[$iterator->address] = $iterator->data;
    }
    return $result;
  }
}

class NetmaskElement {
  var $address;
  var $data;
  var $previous;
  var $next;

  function NetmaskElement($address, $data, &$previous, &$next) {
    $this->address = $address;
    $this->data = $data;
    $this->previous = &$previous;
    $this->next = &$next;
  }
}

?>
