<?php
# Copyright(C) 2004-2007 INL
# Written by Eric Leblond <regit@inl.fr>
#            Vincent Deffontaines <gryzor@inl.fr>
#            Jean Gillaux <jean@inl.fr>
#            Damien Boucard <damien.boucard AT inl.fr>
#
# $Id: operations.php 16897 2008-11-19 15:30:10Z haypo $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
#  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

require_once('elt.class.php');    # max_elt()

function operationSet(&$ruleset, &$object, $link_names)
{
    global $elt, $sub_elt, $title;

    if (!isset($elt) or ($elt==-1)){
        log_error(_('Sorry, bad parameter received'));
        return;
    }
    if (!isset($sub_elt) or ($sub_elt==-1)){
        log_error(_('Sorry, bad parameter received'));
        return;
    }
    $my_elt=&$object->get_elt($elt);
    $GLOBALS['my_elt'] = $my_elt;
    $my_sub_elt=&$my_elt->get_elt($sub_elt);
    $tab = $my_sub_elt->get_obj();
    $received = Array('ID' => $tab{'ID'});
    $seen_mod=0;

    foreach (possible_fields('elt',$tab{'type'}) as $tabkey)
    {
        if ($tabkey!='ID')
            $received{$tabkey}=getHttp($tabkey);
        if ($tabkey=='vpn') {
            if ((!isset($received['vpn']) or ($received['vpn']=='Off'))){
                $received['vpn']='0';
            }else{
                $received['vpn']='1';
            }
        }
        // Empty value which is required?
        if (!isset($received[$tabkey])
            and ($tabkey!='mark') and ($tabkey!='icmptype')
            and ($tabkey!='startday') and ($tabkey!='stopday')
            and ($tabkey!='starthour') and ($tabkey!='stophour')
            and ($tabkey!='sport') and ($tabkey!='dport')
            and ($tabkey!='modified') and ($tabkey!='comment'))
        {
            log_error(sprintf(_('Sorry, at least one parameter is missing (%s).'), $tabkey));
            return;
        }
        if (($received[$tabkey]=='--') and ($tabkey=='action' or $tabkey=='l7rule'))
        {
            $received[$tabkey]='';
        }
        if ($tabkey!='vpn'){
            if (!check_input_var('elt', $tabkey, $received[$tabkey]))
            {
                return;
            }
        }
        if (array_key_exists($tabkey, $tab) and $received[$tabkey] != $tab[$tabkey])
        {
            if (($tabkey!='ID') and ($tabkey!='type') and ($tabkey!='modified'))
            {
                $seen_mod++;
            }
        }
    }

    if (isset($tab['modified']))
    {
        $received['modified']=$tab['modified'];
    }
    $received['type']=$tab['type'];
    unset($received['ID']);

    $received['name'] = strip_magic_slashes($received['name']);

    if ($title == 'applications') {
        $received['ApplicationPath'] = strip_magic_slashes($received['ApplicationPath']);
    } else if ($title == 'resources') {
        if ((isset($received['vpn'])) and ($received['vpn']=='1') and ((!isset($received['mark']) or ($received['mark']=='')) or (!isset($received['net'])) or ($received['net']=='')))
        {
            log_error(sprintf(_('Sorry, at least one parameter is missing (%s).'), 'mark / net'));
            return;
        }
        if ((isset($received['vpn'])) and ($received['vpn']!='1') and (isset($received['mark']) and ($received['mark']!='')))
        {
            log_error(_('Sorry, you specified a parameter that is not valid in this context (').'mark)');
            return;
        }
    }

    if (isset($received['linkid']))
    {
        // Perform a name resolution on received parameter
        if (!isset($link_names))
        {
            $link_names=array();
        }
        $linkid=0;
        foreach ($link_names as $key=>$val){
            if ($val==$received['linkid'])
            {
              if ($key == $my_elt->ID)
              {
                log_error(_("This is not correct: a link cannot point to itself!"));
                return;
              }
              $linkid=$key;
              break;
            }
        }
        if ($linkid==0){
            log_error(_('Sorry, bad linkid parameter received'));
            return;
        }else{
            $received['linkid']=$linkid;
        }
    }
    $my_elt->del_elt($tab['ID']);
    if ($seen_mod){
        $received['modified'] = modifiedTimestamp();
    }
    try {
        $new_elt=new elt($received,'data',$tab['ID']);
        $my_elt->add_elt($new_elt);
        if ($title == 'resources')
        {
            global $used_desc;
            $used_desc = getSession('used_desc', false);
            $descs = $ruleset->descs;
            check_objects_from_desc($object, $descs->elts[$used_desc]);
        }
    } catch (Exception $err) {
        log_error(sprintf(_("Modification error: %s"), $err->getMessage()), $err->getTrace());
        return;
    }
    saveRuleset($ruleset);
}

function operationDel(&$ruleset, &$object)
{
    global $elt, $sub_elt;
    if (!isset($elt) or ($elt==-1)){
        log_error(_('Sorry, bad parameter received'));
        return;
    }
    if (!isset($sub_elt) or ($sub_elt==-1)){
        log_error(_('Sorry, bad parameter received'));
        return;
    }
    $my_elt=&$object->get_elt($elt);
    $my_sub_elt=&$my_elt->del_elt($sub_elt);
    saveRuleset($ruleset);
    $_SESSION['need_acl_recalc']=1;
    $sub_elt = null;
}

function operationDelElt(&$ruleset, &$object_set)
{
    global $title;
    global $elt;
    if (!isset($elt) or ($elt==-1)){
        log_error(_('Sorry, bad parameter received'));
        return;
    }

    // Make sure that the object is not used by another objects
    $object = $object_set->get_elt($elt);
    if (is_a($object, 'l7rule')) {
        $usage = $object->usage($ruleset);
        if ($usage) {
            log_error(sprintf(
                _("The layer 7 object #%s can no be removed. It is used by: %s."),
                $elt,
                $usage));
            return;
        }
    }

    $object_set->del_elt($elt);
    saveRuleset($ruleset);
    deletefromhistory($title, $elt);
    $elt=-1;
}

function operationCopy(&$object)
{
    global $elt, $sub_elt, $floating;
    if (!isset($elt) or ($elt==-1))
    {
        log_error(_('Sorry, need to be passed an element'));
        return;
    }
    if (!isset($sub_elt) or ($sub_elt==-1))
    {
        log_error(_('Sorry, need to be passed a sub element'));
        return;
    }
    $my_elt=$object->get_elt($elt);
    $my_sub_elt=$my_elt->get_elt($sub_elt);
    if (isset($my_sub_elt))
    {
        $my_sub_elt->datas{'ID'}=max_elt($floating)+1;
        array_push($floating,$my_sub_elt);
        $_SESSION['floating_elts']=serialize($floating);
    }
}

function operationSetElt(&$ruleset, &$object, $object_type)
{
    global $elt, $title;

    if (!isset($elt) or ($elt==-1)){
        log_error(_('Sorry, bad parameter received'));
        return;
    }
    $my_elt=&$object->get_elt($elt);
    $tab=$my_elt->get_obj();
    $received['ID']=$tab{'ID'};
    $seen_mod=0;
    foreach (possible_fields($object_type) as $tabkey)
    {
        if ($tabkey!='ID')
            $received{$tabkey}=getHttp($tabkey);
        if ((!isset($received[$tabkey])) and ($tabkey!='modified')){
            log_error(sprintf(_('Sorry, at least one parameter is missing (%s).'), $tabkey));
            return;
        }
        if (!check_input_var('ress', $tabkey, $received[$tabkey]))
        {
            return;
        }
        if (array_key_exists($tabkey, $tab) and ($received[$tabkey] != $tab[$tabkey]))
        {
            $seen_mod++;
            $my_elt->modify_param($tabkey, $received[$tabkey]);
        }
    }
    unset($received['ID']);
    if ($seen_mod)
    {
        $my_elt->modify_param('modified', modifiedTimestamp());
    }
    if ($title == 'resources') {
        $descs = $ruleset->descs;
        $used_desc = getSession('used_desc', false);
        check_objects_from_desc($object, $descs->elts[$used_desc]);
    }
    saveRuleset($ruleset);
}

# Adding an element from the floating panel
function addFloatElements(&$ruleset, &$object)
{
    global $add_float, $add_float_elt, $floating, $sub_elt;

    if (!isset($add_float_elt)){
        log_error(_('Sorry, need to be passed an element'));
        return;
    }

    // We must check passed elements actually exist
    $working_elt=&$object->get_elt($add_float_elt);
    if (!isset($working_elt)){
        log_error(_('Sorry, cannot work on non-existing element'));
        return;
    }
    $exists=1;
    if (!isset($floating))
    {
        $floating=array();
    }
    $reverse_floating = array();
    foreach($floating as $float)
    {
        $reverse_floating[$float->get_id()] = $float;
    }
    foreach($add_float as $id => $quantity)
    {
        if (!isset($reverse_floating[$id])) {
            $exists = 0;
            break;
        }
    }
    if ($exists != 1){
        log_error(_('Sorry, cannot work on non-existing floating element'));
        return;
    }
    foreach ($add_float as $id => $quantity)
    {
        for ($n=0; $n < $quantity; $n++)
        {
            $new_from_float = clone($reverse_floating[$id]);
            $tmp=$new_from_float->get_obj();
            $new_from_float->datas['ID'] = max($object->get_max_elt_id(),max_elt($floating))+1;
            $sub_elt = $new_from_float->datas['ID'];

            if (($tmp['type'] == 'l7rule') or ($tmp['type'] == 'os')
                or ($tmp['type'] == 'app') or ($tmp['type'] == 'duration')
                or ($tmp['type'] == 'nufw') or ($tmp['type'] == 'link')
                or ($tmp['type'] == 'periodicity') or ($tmp['type'] == 'proto')
                or ($tmp['type'] == 'ipv4'))
            {
                $working_elt->add_elt($new_from_float);
                saveRuleset($ruleset);
            }else{
                log_error(_('Sorry, Cannot add an element of that type to resources'));
            }
        }
    }
}

function operationAdd(&$ruleset, &$object, $new_object, $new_name, $class)
{
    global $title, $page;

    if ($new_object != 1){
        log_error(_('Sorry, bad parameter received'));
        return;
    }
    if (!$new_name or !check_input_var('ress', 'name', $new_name))
    {
        return;
    }

    foreach($object->elts as $item)
    {
        if ($item->name == $new_name)
        {
            $url = create_check_element_link($page, $item);
            log_error(sprintf(_('The name "%s" is already used by the element %s, please choose another name!'),
                $new_name, $url));
            return;
        }
    }

    $new_ress{'name'}=$new_name;

    if ($title == 'l7rules')
    {
        $new_ress{'defaultaction'} = 'drop';
        $new_ress{'defaultprefix'} = '';
    }

    try {
        $elt = $object->new_id();
        $new = new $class($new_ress,'data',$elt);
        $object->add_elt($new);
    } catch (Exception $err) {
        log_error(sprintf(_("Unable to create the new element: %s"), $err->getMessage()), $err->getTrace());
        return null;
    }

    saveRuleset($ruleset);
    return $elt;
}

function chooseOperation(&$object, $link_names, $object_type, $class, $new_object)
{
    global $add_float, $add_float_elt;

    $ruleset = $GLOBALS['expolicy'];
    if (getHttp('set') == 1)
    {
        operationSet($ruleset, $object, $link_names);
        return;
    }
    if (getHttp('del') == 1)
    {
        operationDel($ruleset, $object);
        return;
    }
    if (getHttp('elt_set') == 1)
    {
        try {
            operationSetElt($ruleset, $object, $object_type);
        } catch (Exception $err) {
            log_error(sprintf(_("Modification error: %s"), $err->getMessage()), $err->getTrace());
        }
        return;
    }
    if (getHttp('elt_del') == 1)
    {
        operationDelElt($ruleset, $object);
        return;
    }
    if (isset($add_float)){
        addFloatElements($ruleset, $object);
        return;
    }
    if (getHttp('copy') == 1)
    {
        operationCopy($object);
        return;
    }
    $new_name=getHttp('name');
    if ((isset($new_object)) and (isset($new_name)))
    {
        $GLOBALS['elt'] = operationAdd($ruleset, $object, $new_object, $new_name, $class);
        return;
    }

    $populate=getHttp('populate');
    if ($populate=='1' and $add_float_elt)
    {
        quick_add_elt($ruleset, $object, $add_float_elt );
    }
}

function quick_add_elt($expolicy, &$resources, $add_float_elt)
{
    $working_elt=&$resources->get_elt($add_float_elt);
    if (!isset($working_elt)){
        log_error(_('Sorry, cannot work on non-existing element'));
        return;
    }
    $ress=$working_elt->get_obj();
    $name=$ress['name'];
    $dns = getHttp('dns');
    if ($dns == 1) {
        $ips = gethostbynamel($name);
        if (!$ips)
        {
	    log_error(_('A problem occured when performing the DNS lookup. This may be a temporary problem.'));
            return;
        }
        foreach ($working_elt->elts as $key2=>$myres){
            $my_sub_elt=&$working_elt->del_elt($key2);
        }
        foreach ($ips as $ip)
        {
            $received = Array(
                'type' => 'ipv4',
                'name' => 'from_dns_'.$ip,
                'net' => $ip);
            $new_elt=new elt($received,'data',$resources->get_max_elt_id()+1);
            $working_elt->add_elt($new_elt);
            $GLOBALS['sub_elt'] = $new_elt->get_id();
        }
    }else{
        $new_elt_type=getHttp('elt_type');
        if (!$new_elt_type)
            return;
        $elttype = $new_elt_type;
        if (($elttype != "link") and ($elttype != "ipv4") and ($elttype!="nufw") and ($elttype!="proto") and ($elttype!="app") and ($elttype!="os") and ($elttype!="l7rule") and ($elttype!="periodicity") and ($elttype!="duration"))
        {
            add_log(sprintf(_('Wrong type received: "%s". Invalid request!'), $elttype));
            return;
        }
        $name = getHttp($new_elt_type);
        $received = array(
            'type' => $new_elt_type,
            'name' => $name,
        );
        if ($new_elt_type == "link")
            $received['linkid'] = $name;
        if ($new_elt_type == "ipv4")
            $received['net'] = $name;
        if ($new_elt_type == "proto")
        {
            $received['proto'] = $name;
            $received['dport'] = getHttp('dport');
            $received['sport'] = getHttp('sport');
            $received['icmptype'] = getHttp('icmptype');
            $received['name'] = $name." ".getHttp('dport');
            if ($name != 'icmp')
                $received['icmptype'] = '--';
            if (($name != 'tcp') and ($name != 'udp'))
            {
                $received['sport'] = '';
                $received['dport'] = '';
            }
	    if (!check_input_var('elt','icmptype',$received['icmptype']))
	    {
	      return;
	    }
        }
	if ($new_elt_type == "nufw")
	{
	  $received['group'] = getHttp('group_select');
          if (!isset ($name))
          {
            $received['name'] = $received['group'];
          }
	}
	if ($new_elt_type == 'os')
	{
	  $received['OSName'] = getHttp('OSName');
	  $received['OSRelease'] = getHttp('OSRelease');
	  $received['OSVersion'] = getHttp('OSVersion');
	  $received['name'] = $received['OSName'].'-'.$received['OSVersion'];
	  if (!check_input_var('elt','OSName',$received['OSName']))
	    return;
	  if (!check_input_var('elt','OSRelease',$received['OSRelease']))
	    return;
	  if (!check_input_var('elt','OSVersion',$received['OSVersion']))
	    return;
	}
	if ($new_elt_type == 'app')
	{
          $path = getHttp('ApplicationPath');
          $path = strip_magic_slashes($path);
          $received['ApplicationPath'] = $path;
          $received['name'] = $path;
	  if (!check_input_var('elt','ApplicationPath', $received['ApplicationPath']))
	    return;
	}
	if ($new_elt_type == 'periodicity')
	{
	  $received['starthour'] = '8';
	  $received['stophour'] = '18';
	  $received['startday'] = '1';
	  $received['stopday'] = '5';
	}
	if ($new_elt_type == 'duration')
	{
	  $received['duration'] = getHttp('duration');
	  $received['name'] = $received['duration']._("seconds");
	  if (!check_input_var('elt','duration',$received['duration']))
	    return;
	}
        $elt_id = $resources->get_max_elt_id()+1;

        if (!check_name($received['name'], 'name')) {
            return;
        }

        try {
            $new_elt=new elt($received,'data', $elt_id);
            $working_elt->add_elt($new_elt);
        } catch (Exception $err) {
            log_error(sprintf(_("Unable to create the new element: %s"), $err->getMessage()), $err->getTrace());
            return;
        }
        $GLOBALS['sub_elt'] = $elt_id;
    }
    saveRuleset($expolicy);
}

?>
