<?php
# Copyright(C) 2004-2008 INL
# Written by Eric Leblond <regit@inl.fr>
#            Vincent Deffontaines <gryzor@inl.fr>
#            Jean Gillaux <jean@inl.fr>
#            Damien Boucard <damien.boucard AT inl.fr>
#
# $Id: page_function.php 17391 2009-01-08 15:02:25Z haypo $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 of the License.
#
#  This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

require_once("ruleset.class.php");
require_once("menu.php");
require_once("history.php");
require_once('html.php');

$page_new_button = Array(
    'periodicities' => _('New periodicity'),
    'durations' => _('New duration'),
    'resources' => _('New network resource'),
    'auths' => _('New authentication'),
    'protocols' => _('New protocol'),
    'l7rules' => _('New layer7 filter'),
    'applications' => _('New application'),
    'localsin' => _('New INPUT rules'),
    'localsout' => _('New OUTPUT rules'),
    'os' => _('New operating system'),
    'acls' => _('New ACL'),
);

$help_matrix = Array(
    'name' => _("The name of an element is set by you, and should represent the administrative definition of the object. Allowed characters are letters, spaces and numbers. The name of an element can be changed at any time."),
    'ID' => _("The ID of an element is set internally by NuFace, and cannot be modified, unless you delete the element. Elements ID are all numeric, and NuFace guarantees that they are unique for the ACLs set you are working on."),
    'type' => _("The type of an element is a marker that defines what contents this element can host. Available types are : <ul><li>proto : for the protocols page</li><li>nufw : for the authentication page</li><li>ipv4 : for the resources page</li><li>application</li><li>os : for the operating systems page</li><li>periodicity</li><li>duration</li><li>link : available on all pages in order to link to other objects of the page</li></ul>"),
    'modified' => _("The <i>modified</i> attribute of all NuFace elements contains the date of last modification of the element. There is no way for the administrator of changing it, unless you actually apply modifications to the element."),
    'proto' => _("On the protocols page, you can set the protocol for each element. Protocol can be defined as TCP, UDP, ICMP, ESP or AH. TCP is the most often used protocol ; UDP is used for many applications with real time needs, such as VoIP, and ICMP is used for sending control and error messages. ESP and AH protocols are only used for IPSEC Virtual Private Networks.<br />If you choose TCP or UDP, NuFace will let you choose source and destination ports. If you choose ICMP, you will be able to specify an optional ICMP type."),
    'dport' => _("If you use TCP or UDP as protocol, you can choose the destination port that you are willing to set. You can find a list of port numbers <a href=\"http://en.wikipedia.org/wiki/List_of_TCP_and_UDP_port_numbers\" target=\"new\">on Wikipedia</a>. This parameter must be a number, between 1 and 65535. You can also set a range by specifying, for example <i>21:23</i>. This will mean :\"Any number between 21 and 23, including the bounds.\""),
    'sport' => _("If you use TCP or UDP as protocol, you can choose the source port that you are willing to set. This parameter must be a number, between 1 and 65535. You can also set a range by specifying, for example <i>1024:65535</i>. This will mean :\"Any number between 1024 and 65535, including the bounds.\". Most of the time, source ports are numbers between 1024 and 65535, so <i>1024:65535</i> should probably be used here. You can also leave this field empty which means \"Any source port\". Only set specific source ports if you know exactly what you are doing."),
    'icmptype' => _("If you use ICMP as protocol, you can choose the ICMP type that you are willing to set. This parameter must be chosen from the list provided by NuFace. The ICMP type may also be left blank, which means \"Any ICMP datagram\". A list and description of ICMP types is available at <a href=\"http://en.wikipedia.org/wiki/Internet_Control_Message_Protocol\" target=\"new\">Wikipedia</a>"),
    'l7rule' => _("When setting up protocols, the l7rule parameter is suggested. This refers to the Layer 7 filtering rules that you may have defined on the l7rules page. Layer 7 filtering inspects connections travelling through the firewall and takes decision based on the connection contents."),
    'group' => _('For Authentication objects (type nufw), the group should always be specified. The group is a number that suits your users directory groups scheme. For instance, usually, administrators group is the group number 513. If you set the $use_getent_group in NuFace config file, you will get directly a list of your actual named groups, and you will just have to pick your group in the list.'),
    'net' => _('Resources elements that are of type IPv4 contain a <i>net</i> attribute, which must contain a network object. A network object is either an IP address (<i>192.168.33.1</i>), or a network address (such as <i>10.0.0.0/255.0.0.0</i> or in CIDR format <i>10.0.0.0/8</i>). The Internet is always defined as <i>0.0.0.0/0</i> and is auto-generated by NuFace if it does not exist when opening the ACLs file. After you create or modify a network object, NuFace displays the Network name the new element belongs to, according to your network topology.'),
);

function help_available($name){
  global $help_matrix;
  if (array_key_exists($name, $help_matrix))
    return $help_matrix{$name};
  return null;
}

function begin_content_detail(){
  echo '<div class="content">';
}

function end_content_detail(){
  echo '</div> <!-- content -->';
}

function begin_acls_use(){
  echo '<div class="acl_use">';
}

function end_acls_use(){
  echo "</div> <!-- acl_use -->\n";
}


function end_page(){
  require_once ('footer.php');
}

function initFloatingElts()
{
    global $floating;
    if (array_key_exists('floating_elts', $_SESSION)) {
        $floating = unserialize($_SESSION['floating_elts']);
        if (!isset($floating) or ($floating == ''))
        {
            $floating = null;
        }
    } else {
        $floating = null;
    }
}

function getAddFloat() {
    global $add_float, $add_float_elt;

    $add_float_elt=getHttp('elt');

    $add_float=array();
    foreach ($_REQUEST as $req=>$val)
    {
        if (preg_match("/^add_float_(\d+)$/",$req,$matches)){
            $add_float[$matches[1]]=$val;
        }
    }
    if (count($add_float)==0)
    {
        $add_float = null;
    }

    if (!check_array_of_nb($add_float))
    {
        log_error(_('Sorry, bad parameter received'));
        log_and_exit(-1);
    }

    if (!check_nb($add_float_elt))
    {
        log_and_exit(-1);
    }
}

function getSearch()
{
    global $new_search;

    $new_search=getHttp('search');
    if (!check_search($new_search))
    {
        log_error(_('Sorry, bad parameter received')." : search");
        log_and_exit(-1);
    }

    if (preg_match("/^\s*$/",$new_search))
    {
        $new_search = null;
    }
}

function searchResultStart()
{
    global $new_search, $title;
    print "<div class=\"menu\">\n";
    print '<div class="filterbox">';
    print "<form action=\"$title.php\" method=POST>";
    print '<input type="text" name="search" value="'.htmlspecialchars($new_search).'">';
    print " <input type=submit value=\""._("Filter")."\" class=\"button\">";
    print "</form>";
    print '</div>';
    print "<img src=\"images/collapse.png\" onClick=\"hide_all_objects();\" title=\""._('Collapse all')."\" alt=\"[-]\">";
    print "&nbsp;";
    print "<img src=\"images/expand.png\" onClick=\"show_all_objects();\" title=\""._('Expand all')."\" alt=\"[+]\">";
    print "<ul id=\"containerul\">\n";
}

function searchResultEnd()
{
    global $elt;

    print "</ul>";
    menuJS($elt);
    print "</div>";
}

function searchPrepare($ress)
{
    $ress_array = array();
    $seen = array();
    foreach ($ress as $key=>$res){
        if (array_increment($seen, $res)<1)
            array_push($ress_array,$res);
    }
    natcasesort($ress_array);
    return $ress_array;
}

function searchHeader(&$object, $tmp, $res, $myress)
{
    global $new_search;

    $infos=$tmp->get_obj();
    if (!isset($new_search))
        return 1;
    $regex = "/$new_search/i";
    if (preg_match($regex,$infos{'name'})>0)
      return 1;

    if (!isset($myress)) {
        return 0;
    }
    foreach ($myress as $key=>$myres){
        $my_elt=$tmp->get_elt($key);
        $tab=$my_elt->get_obj();
        if (preg_match($regex, $tab{'name'})>0)
        {
            return 1;
        }
        if (isset($tab{'net'}) and preg_match($regex, $tab{'net'})>0)
        {
            return 1;
        }
        if (isset($tab{'dport'}) and preg_match($regex, $tab{'dport'})>0)
        {
            return 1;
        }

    }
    return 0;
}

function searchShow($display_callback, $res, $tmp, $key, $link_names, $data)
{
    global $elt, $used, $used_link, $title, $new_search;
    echo '<li id="elt_'.$key.'">';
    if ($elt == $key)
        print "<div class=\"highlightelt\">";

    if (isset($used{$key}))
        print '<span class="used">';
    elseif (isset($used_link{$key}))
        print '<span class="usedaslink">';

    print '<a href="'.$title.'.php?elt='.$key;
    if (isset($new_search))
        print "&search=$new_search";
    print '">'.$res.'</a>';

    if ((isset($used{$key})) or (isset($used_link{$key})))
        print "</span>\n";

    searchResultDisplay($display_callback, $tmp, $key, $link_names, $data);

    if ($elt == $key)
        print "</div> <!-- highlightelt -->\n";
    print "</li>";
}

function displayProtocolAttr(&$text, $tab)
{
    if (isset($tab{'proto'}))
        $text[] = "proto: ".$tab{'proto'};
    if ((isset($tab{'sport'})) and ($tab{'sport'}!=''))
        $text[] = "sport: ".$tab{'sport'};
    if ((isset($tab{'dport'})) and ($tab{'dport'}!=''))
        $text[] = "dport: ".$tab{'dport'};
}

function displayResourceAttr(&$text, $tab)
{
    if (isset($tab{'net'}))
        $text[] = "net: ".$tab{'net'};
}

function displayL7Attr(&$text, $tab)
{
    if (isset($tab{'l7proto'}))
        $text[] = "l7proto: ".$tab{'l7proto'};
    if ((isset($tab{'action'})) and ($tab{'action'}!=''))
        $text[] = "dport: ".$tab{'action'};
}

function displayApplicationAttr(&$text, $tab)
{
    if (isset($tab{'ApplicationPath'}))
        $text[] = "ApplicationPath: ".$tab{'ApplicationPath'};
}

function displayOsAttr(&$text, $tab)
{
    if (isset($tab{'OSName'}))
        $text[] = "OSName: ".$tab{'OSName'};
    if (isset($tab{'OSRelease'}))
        $text[] = "OSRelease: ".$tab{'OSRelease'};
    if (isset($tab{'OSVersion'}))
        $text[] = "OSVersion: ".$tab{'OSVersion'};
}

function displayAuthAttr(&$text, $tab)
{
    if (isset($tab{'group'}))
        $text[] = "group: ".$tab{'group'};
}

function displayPeriodicityAttr(&$text, $tab)
{
    if ((isset($tab{'startday'})) and ($tab{'startday'}!=''))
        $text[] = "startday: ".$tab{'startday'};
    if ((isset($tab{'stopday'})) and ($tab{'stopday'}!=''))
        $text[] = "stopday: ".$tab{'stopday'};
}

function displayDurationAttr(&$text, $tab)
{
    if (isset($tab{'duration'}))
        $text[] = "duration: ".$tab{'duration'};
}

function searchResultDisplayItem($display_callback, $tmp, $elt, $tmp_key, $link_names, $text)
{
    global $title, $new_search;

    $my_elt = $tmp->get_elt($tmp_key);
    if(!isset($my_elt)){
        # FIXME: Remove the echo
        echo "<li>DEBUG : EMPTY TMP_KEY</li>";
        return;
    }

    echo "<li>";

    echo "<a href=\"$title.php?elt=$elt&amp;sub_elt=$tmp_key";
    if (isset($new_search))
        print "&search=$new_search";
    print '"';

    $tab=$my_elt->get_obj();
    if (isset($tab)){
        $a_title = Array();
        if ($display_callback)
            $display_callback($a_title, $tab);
        if ($a_title)
            $a_title = ' '.implode(", ", $a_title);
        else
            $a_title = '';
        print ' title="['.$tab{'type'}.']'.$a_title.'"';
    }
    if (isset($my_elt->datas['action']) and ($my_elt->datas['action']!='')) {
        // CSS class used by l7rules
        print ' class="'.$my_elt->datas['action'].'"';
    }
    echo ">";

    echo $text;
    if (isset($tab) and $tab['type']=='link' and isset($tab{'linkid'}))
    {
        print " (".$link_names[$tab{'linkid'}].")";
    }

    echo "</a></li>\n";
}

function searchResultDisplay($display_callback, $tmp, $elt, $link_names, $data)
{
    if (!isset($data)) return;
    echo "<ul>\n";
    foreach ($data as $tmp_key=>$text){
        searchResultDisplayItem($display_callback, $tmp, $elt, $tmp_key, $link_names, $text);
    }
    echo "</ul>";
}

function elementIsUsed($elt)
{
    global $used, $used_link;
    return  isset($used['acls']{$elt})
         or isset($used['localsin']{$elt})
         or isset($used['localsout']{$elt})
         or isset($used_link{$elt});
}

function deleteFormMain($elt, $array_types, $title)
{
    if (isset($array_types))
    {
        global $expolicy;

        begin_acls_use();
        echo "<h3>"._("In use in ACLs:")."</h3>";
        show_acls_use($expolicy, $array_types, $elt, _("Not used by any ACL"));
        echo "<h3>"._("In use as link by:")."</h3>";
        show_links_use($expolicy, $title, $elt, _("Not used as a link"));
        end_acls_use();
    }
}

# Display search result
# $display_callback prototype: function display(&$text, $tab)
function searchAll(&$object, $display_callback)
{
    $ress = $object->list_tab();

    searchResultStart();
    $ress_array = searchPrepare($ress);
    foreach($ress_array as $res)
    {
        foreach($ress as $key=>$foo)
        {
            if ($res == $foo)
                break;
        }
        $tmp = &$object->get_elt($key);
        if (!isset($tmp)){
            continue;
        }
        $myress=$tmp->list_tab();

        $show_this = searchHeader($object, $tmp, $res, $myress);
        if ($show_this == 1)
        {
            searchShow($display_callback, $res, $tmp, $key, $ress, $myress);
        }
    }
    searchResultEnd();
}

function display_getent($select_name, $groups, $selected=null)
{
    asort($groups);
    $groups2 = Array();
    $found = false;
    foreach ($groups as $gid=>$name)
    {
        $groups2[$gid] = "$name ($gid)";
        if ($gid == $selected)
            $found = true;
    }
    if ($selected and !$found)
        $groups2[$selected] = $selected;
    select_array($select_name, $groups2, $selected);
}

function editItem($link_names, $tab, $tabkey)
{
    global $types, $title, $use_getent_groups;

    $array_mess_daysweek = array(
        _('Monday'), _('Tuesday'), _('Wednesday'),
        _('Thursday'), _('Friday'), _('Saturday'), _('Sunday'));

    if ($tabkey=='vpn'){
        return;
    }

    $tabelt='';
    if (isset($tab[$tabkey]) and $tab[$tabkey]!='')
        $tabelt = $tab[$tabkey];

    print '<tr>';
    print '<td class="rowtitle">';
    if ($tabkey=='mark')
    {
        $disabled='onFocus="griser_mark()" onchange="griser_mark()"';
        $value="";
        if ($tab{'vpn'}==1)
            $value="checked";
        print 'vpn</td>';
        print '<td>'.$tab{'vpn'}."</td><td><input type=checkbox name=\"vpn\" ".$value." $disabled>";
    }
    if ($tabkey=='comment')
    {
        print 'Comment</td>';
        print '<td colspan="2">';
        html_textarea("comment", $tabelt, 40, 2);
    }else if ($tabkey == 'l7rule'){
        global $l7_firewall;
        global $expolicy;
        if ($l7_firewall){
            print "$tabkey</td>";
            $l7rules = $expolicy->l7rules;
            $ret='';
            foreach (($l7rules->list_tab()) as $key=>$l7)
            {
                if ($key == $tabelt)
                {
                    $ret=$l7;
                }
            }
            print "<td>$ret</td>";

            $array_l7 = array();
            foreach (($l7rules->list_tab()) as $key=>$l7)
            {
                array_push($array_l7,$l7);
            }
            natcasesort($array_l7);

            print "<td><select name=\"$tabkey\">\n";
            print "<option value=\"--\">--</option>\n";
            foreach ($array_l7 as $l7)
            {
                foreach ($l7rules->list_tab() as $key=>$foo)
                {
                    if ($foo==$l7)
                        break;
                }
                if ($key!=""){
                    print "<option value=\"$key\"";
                    if ($key == $tabelt)
                        echo " selected";
                    print ">".$l7."</option>";
                }
            }
            print " </select>\n";
        }
    }else if (($tabkey == 'group') and $use_getent_groups){
        print "$tabkey</td><td>";
        $array_getent = getent_user_groups();
        if (isset($tabelt)){
          if (isset($array_getent[$tabelt]))
            print $array_getent[$tabelt];
          else
            print $tabelt;
        }
        print "</td><td>\n";

        display_getent($tabkey, $array_getent, $tabelt);
    }else{
        // Row title
        print "$tabkey</td>";

        //This field can be edited?
        $editable = (isset($types{'elt'}{$tabkey}) and $types{'elt'}{$tabkey}{'edit'} == 1);
        if ($editable)
            print '<td>';
        else
            print '<td colspan="2">';

        // Current value
        if ($tabkey == 'linkid') {
            print "<a href=\"$title.php?elt=".$tabelt."\">".$link_names[$tabelt]."</a>";
        } else if (($tabkey == 'startday') or ($tabkey == 'stopday')) {
            print $array_mess_daysweek{$tabelt-1};
        } else {
            print $tabelt;
        }

        if ($editable)
        {
            print "</td><td>";
            $disabled='';
            if (($tabkey=='mark')
                or ($tabkey=='net') or ($tabkey=='zone')
                or ($tabkey=='periodtype')
               )
            {
                $disabled='onFocus="griser_mark();" onChange="griser_mark();"';
            } else if ($tabkey == 'proto') {
                $disabled='onLoad="griser_type();" onFocus="griser_type();" onChange="griser_type();"';
            } else if ($tabkey == 'periodtype') {
                $disabled='onFocus="griser_period();" onChange="griser_period();"';
            } else if ($tabkey=='name') {
                $disabled='onFocus="remove_dummy_name()"';
            }

            if ($tabkey=='icmptype')
                $possible_values=possible_values('proto', 'icmptype');
            else
                $possible_values=possible_values('elt', $tabkey);
            if ($possible_values==0) //No preconfigured value for this field
            {
                echo "<input type=\"text\" name=\"$tabkey\" value=\"$tabelt\" $disabled>\n";
            }else{

                if ($possible_values
                    and ($title != "l7rules" and $title != "protocols"))
                {
                    natcasesort($possible_values);
                }

                echo "<select name=\"$tabkey\" $disabled>\n";
                foreach ($possible_values as $value)
                {
                    $value = preg_replace('/\n/','',$value);
                    print "  ";
                    if (($tabkey=='startday') or ($tabkey == 'stopday'))
                        print "<option value=\"$value\"";
                    else
                        print "<option";
                    if (($value == $tabelt)
                        or (($tabkey=='linkid') and ($value == $link_names[$tabelt]))
                       )
                        echo " selected>";
                    else
                        echo ">";

                    if (($tabkey=='startday') or ($tabkey == 'stopday'))
                    {
                        print $array_mess_daysweek{$value-1};
                    }else{
                        print $value;
                    }

                    print "</option>\n";
                }
                echo "</select>\n";
            }
        }
    }
    $help = help_available($tabkey);
    if ($help)
    	contextual_help($tabkey, $help);
    echo "</td></tr>\n";
}

function editForm($header_title, $link_names, $my_elt, $sub_elt, $form_name="")
{
    global $title;

    $elt_obj = $my_elt->get_obj();
    $my_sub_elt=$my_elt->get_elt($sub_elt);
    $tab=$my_sub_elt->get_obj();

    print "<h2>$header_title ".$elt_obj{'name'}."</h2>\n";
    if (array_key_exists('name', $tab)) {
        $name = $tab['name'];
    } else {
        $name = '';
    }
    print "<h3>".sprintf(_('Edition of the element "%s"'), $name);
    editDelete(false, $my_elt, $sub_elt);
    echo "</h3>\n";

    print "<form action=\"$title.php\" method=\"post\"";
    // FIXME: $form_name est important ? Je pense qu'il peut etre supprime
    if ($form_name!="")
    {
      print " name=\"$form_name\"";
    }
    print ">\n";

    print '<input type="hidden" name="set" value="1">'."\n";
    formCommonFields();
    print '<input type="hidden" name="sub_elt" value="'.$sub_elt.'">'."\n";

    print "<table>\n";
    print "<tr class=\"head\"><td>"._('Field')."</td><td>"._('Value')."</td><td>"._('New Value')."</td></tr>\n";
    print "<tr><td class=\"rowtitle\">type</td><td colspan=2>".$tab{'type'};
    contextual_help('type',help_available("type"));
    print "</td></tr>\n";
    foreach (possible_fields('elt',$tab{'type'}) as $tabkey)
    {
        if ($tabkey == 'ID')
            continue;
        editItem($link_names, $tab, $tabkey);
    }
    echo "</table><input type=submit value=\""._('Save')."\" class=\"button\" onMouseOver=\"griser_type();\" >";
    echo "</form>\n";
    if (($title == 'protocols') and ($tab['type'] != 'link')) {
        html_javascript('griser_type();', true);
    }
}

function editMainItem($link_names, $tab, $tabkey)
{
    global $types;
    $tabelt='';
    if (array_key_exists($tabkey, $tab))
        $tabelt = $tab{$tabkey};
    print "<tr>";
    print "<td class=\"rowtitle\">$tabkey</td>";

    if (array_key_exists($tabkey, $types{'ress'}))
        $editable = ($types{'ress'}{$tabkey}{'edit'} == 1);
    else
        $editable = false;
    if ($editable)
    {
        echo "<td>$tabelt</td>";
        print "<td>";
        $possible_values=possible_values('ress',$tabkey);
        if ($possible_values==0) //No preconfigured value for this field
        {
            echo "<input type=\"text\" name=\"$tabkey\" value=\"$tabelt\">\n";
        }else{
            echo "<select name=\"$tabkey\">\n";
            $possible_values and natcasesort($possible_values);
            foreach ($possible_values as $value)
            {
                print "  <option";
                if (($value == $tabelt) or (($tabkey=='linkid') and ($value==$link_names[$tabelt])))
                    echo " selected";
                print ">$value</option>\n";
            }
            echo "</select>\n";
        }
        print "</td>";
    } else {
        echo '<td colspan="2">'.$tabelt."</td>";
    }
    print "</tr>\n";

}

function editDelete($used, $my_elt, $sub_elt=null)
{
    global $title;
    if (!$used) {
        $msg = _('Are you sure that you want to delete this element?');
        $url = "$title.php?elt=".$my_elt->ID;
        if (!is_null($sub_elt)) {
            $url .= "&sub_elt=$sub_elt&del=1";
        } else {
            $url .= '&elt_del=1';
        }
        deleteIcon($url, _("Delete the element"), $msg);
    } else {
        $text = sprintf(_("Element can not be deleted (used by an ACL or an element)"), $my_elt->str());
        $alt = sprintf("(%s)", _('Delete disabled'));
        echo ' ';
        html_tag("img", Array('src' => 'images/remove_grey.png', 'alt' => $alt, 'title' => $text), null);
    }
}

function editMain($header_title, $link_names, $my_elt, $attributes=Array())
{
    global $title;
    $tab=$my_elt->get_obj();
    print "<h3>";
    echo sprintf(_('Edition of the element "%s"'), $tab['name']);
    editDelete(elementIsUsed($my_elt->ID), $my_elt);
    echo "</h3>\n";
    print "<table><tr class=\"head\"><td>"._('Field')."</td><td>"._('Value')."</td><td>"._('New Value')."</td></tr>\n";
    print "<form action=\"$title.php\" method=POST>\n";
    formCommonFields();
    print "<input type=hidden name=elt_set value=1>\n";

    $attributes = array_merge(Array('ID', 'name'), $attributes, Array('modified'));
    foreach($attributes as $tabkey){
        if ($tabkey != 'ID')
        editMainItem($link_names, $tab, $tabkey);
    }

    print "</td></tr><tr><td class=\"rowtitle\">Comment</td><td colspan=\"2\">";
    html_textarea("comment", $tab['comment'], 30, 2);
    echo "</td></tr></table>";
    print "<input type=submit value=\""._('Save')."\" class=\"button\">";
    print "</form>\n";
}

function editContent($header_title, $object, $link_names,
                     $elt, $sub_elt, $array_types, $attributes=Array())
{
    global $title, $in_elt;
    global $singtitle, $page_new_button;

    $show_warning = false;

    $my_elt = null;
    $my_sub_elt = null;
    if (isset($elt) and ($elt!=-1)) {
        // Get element
        if (!$object->has_elt($elt)) {
            log_error(sprintf(_("There is no element with identifier %s!"), $elt));
            return;
        }
        $my_elt=$object->get_elt($elt);

        if ((isset($sub_elt)) and ($sub_elt!=-1)) {
            // Get sub-element
            if (!$my_elt->has_elt($sub_elt)) {
                log_error(sprintf(_("There is no sub-element with identifier %s!"), $sub_elt));
                return;
            }
            $my_sub_elt = $my_elt->get_elt($sub_elt);
        }

    }

    begin_content_detail();
    print $page_new_button{$title};
    print "<form action=\"$title.php\" method=\"post\">";
    html_input_text('name', '', Array('size' => 20));
    echo ' ';
    html_hidden("new_$singtitle", 1);
    html_submit(_('New'), Array('class' => 'button'));
    print "</form>";

    if ($my_elt)
    {
        print "<div class=\"edit_tab\">\n";
        $form_name="";
        if ($array_types=='proto')
        {
          $form_name='type';
        }
        if ($my_sub_elt) {
            editForm($header_title, $link_names, $my_elt, $sub_elt, $form_name);
            if ($my_sub_elt->datas['type']=='ipv4')
              netResume($elt,$sub_elt);
        } else {
            $sub_elts = $object->elts[$elt]->elts;
            if (count($sub_elts)==0)
              $show_warning = true;
            $in_elt=1;
            editMain($header_title, $link_names, $my_elt, $attributes);
            if ($title == 'resources')
            {
                print "<form action=\"resources.php\" method=POST>\n";
                print "<input type=hidden name=elt value=$elt>\n";
                print "<input type=hidden name=populate value=1>\n";
                print "<input type=hidden name=dns value=1>\n";
                print "<input type=submit value=\""._('Populate from DNS')."\" class=\"button\">";
                print "</form>";
            }
            add_create_subelt_button($title, $link_names, $show_warning);
            deleteFormMain($elt, $array_types, $title);
        }
        print "</div>\n";
    }
    end_content_detail();
}

function add_create_elt_button($elt_type)
{
    yahoo_ask_help($elt_type, true);

    print "<div id=\"ask-help-$elt_type\" class=\"Newbutton\"><a>";
    print _("New");
    print "</a></div>\n";
}

function add_js_form($page_name, $elt_type)
{
  print "<div id=\"contexthelp$elt_type\" class=\"nufacehelp\">\n";
  print "<div class=\"hd\">New $elt_type</div>\n";
  print "<div class=\"bd\">\n";
  print "<form action=\"$page_name.php\" method=POST id=\"js_$elt_type\">\n";
  print "<input type=\"hidden\" name=\"new_$elt_type\" value=1>\n";
  print "<input type=\"text\" name=\"name\">\n";
  print "<input type=submit>\n";
  print "</form>\n";
  print "</div>\n";
  print "</div>\n";
}

function add_create_subelt_button($title,$link_names,$show_warning=false)
{
  global $use_getent_groups;
  global $elt;

  $types=array();
  if ($title == "resources")
    array_push($types, Array("ipv4", _("New IPv4 address")));
  if ($title == "protocols")
    array_push($types, Array("proto", _("New protocol")));
  if ($title == "auths")
    array_push($types, Array("nufw", _("New authentication")));
  if ($title == "os")
    array_push($types, Array("os", _("New operating system")));
  if ($title == "applications")
    array_push($types, Array("app", _("New application")));
  if ($title == "periodicities")
    array_push($types, Array("periodicity", _("New periodicity")));
  if ($title == "durations")
    array_push($types, Array("duration", _("New duration")));
  if ($title == "l7rules")
    array_push($types, Array("l7rule", _("New layer7 rule")));
  array_push($types, Array("link", _("New link")));

  $displayed_warning = false;

  foreach ($types as $data)
  {
      list($object, $new_text) = $data;

      if (($show_warning) and (! $displayed_warning))
      {
        print "<div class=info>";
        print _("You need to fill this object with elements, click one of these:");
        print "</div>\n";
        $displayed_warning = true;
      }

      if (($object == "link") and count($link_names) < 2) {
          continue;
      }

      if ($object != "link")
      {
          $text = null;
          if ($title == "protocols") {
              $text = _("New protocol");
              $href = "$title.php?elt=$elt&populate=1&elt_type=proto&proto=tcp&dport=80";
          }
          if ($title == "periodicities") {
              $text = _("New periodicity");
              $href = "$title.php?elt=$elt&populate=1&elt_type=periodicity";
          }
          if ($text) {
              echo "<div class=\"addelt\" id=\"ask-help-proto\">";
              echo "<a href=\"$href\">$text</a>";
              echo "</div>\n";
              continue;
          }
      }

    yahoo_ask_help($object);

      print "<div id=\"contexthelp$object\" class=\"nufacehelp\">\n";
      print "<div class=\"hd\">$new_text</div>\n";
      print "<div class=\"bd\">\n";
      print "<form action=\"$title.php\" method=POST id=\"js_$object\">\n";
      print "<input type=\"hidden\" name=\"elt\" value=\"$elt\">\n";
      print "<input type=\"hidden\" name=\"populate\" value=1>\n";
      print "<input type=\"hidden\" name=\"elt_type\" value=\"$object\">\n";
      print "<table>";

      if ($object =="link")
      {
          print "<select name=$object>\n";
          $disp_array = array();
          foreach ($link_names as $id=>$link)
          {
              if ($id != $elt)
              {
                  $disp_array{$id}=$link;
              }
          }
          natcasesort($disp_array);
          foreach ($disp_array as $id=>$link)
          {
              print "  <option value=\"$id\" label=\"$link\">$link</option>\n";
          }
          print "</select>\n";
      } else if ($object == 'ipv4'){
          print _("New address: ")."<input type=text name=$object>\n";
      }else if ($object == 'proto'){
          print "<tr><td>";
          print _("Protocol");
          print "</td><td><select name='proto'>\n";
          foreach(possible_values('elt','open_proto') as $t_proto)
          {
              print "  <option";
              if ($t_proto == 'tcp')
                  print " selected";
              print ">$t_proto</option>\n";
          }
          print "</select></td></tr>\n";
          print "<tr><td>";
          print _("Destination port");
          print "</td><td><input type=\"text\" name=\"dport\" value=\"80\"></td></tr>\n";
          print "<tr><td>";
          print _("Source port");
          print "</td><td><input type=\"text\" name=\"sport\" value=\"1024:65535\"></td></tr>\n";
          print "<tr><td>Icmptype</td><td><select name='icmptype'>\n";
          foreach(possible_values('proto','icmptype') as $t_type)
          {
              print "  <option>$t_type</option>\n";
          }
          print "</select></td></tr>\n";

      }else if ($object == 'nufw'){
          if ($use_getent_groups)
          {
              echo '<tr><td class="rowtitle">'._('User group: ').'</td><td>';
              $users = getent_user_groups();
              display_getent('group_select', $users);
              echo "</td></tr>";
              $groups_shown = true;

          }else{ #no getentnufw
              echo '<tr><td>name</td><td><input type=text name="nufw"></td></tr>';
              echo '<tr><td>group</td><td><input type=text name=group_select"></td></tr>';
          }
      }else if ($object =='os'){
	print "<tr><td>";
	print _("OS");
	print "</td><td><input type=text name=\"OSName\" value=\"linux\"></td></tr>";
	print "<tr><td>";
	print _("Release");
	print "</td><td><input type=text name=\"OSRelease\" value=\"2.6\"</td>";
	print "</tr>";
	print _("Version");
	print "</td><td><input type=text name=\"OSVersion\" value=\"2.6\"</td>";
	print "</tr>";
      }else if ($object == 'app'){
	print _("Application Path");
	print "<input type=text name=\"ApplicationPath\" value=\"/usr/bin/firefox\">";
      }else if ($object == 'duration'){
	print _("Duration");
	print "<input type=text name=duration value=10>";
	print _("seconds");
      }
      print " <tr><td></td><td><input type=submit></td></tr>\n";
      print "</table>";
      print "</form>\n";
      print "</div>\n";
      print "</div>\n";
      print "<div class=\"addelt\" id=\"ask-help-$object\"><a>$new_text</a></div>\n";
  }
}
function formCommonFields()
{
    global $elt, $sub_elt, $new_search;

    print '<input type="hidden" name="elt" value="'.$elt.'">'."\n";
    if (isset($new_search))
      print '<input type="hidden" name="search" value="'.$new_search.'">'."\n";
}

function netResume($elt,$sub_elt)
{
    $document = unserialize($_SESSION['document']);
    $used_desc = $_SESSION['used_desc'];
    $resources = $document->resources;
    $descs = $document->descs;
    $my_elt=$resources->get_elt($elt);
    $my_sub_elt = $my_elt->get_elt($sub_elt);
    $descnet = gen_descnets($descs->elts[$used_desc]);

    if (!array_key_exists('net', $my_sub_elt->datas))
        return;
    $net = $my_sub_elt->datas['net'];
    $net_array = array($net);
    $in_desc = fill_seen_tab($net_array, $descnet);
    if (!$in_desc)
        return;

    $in_desc = $in_desc[0];
    $name = $descs->elts[$used_desc]->networks->elts[$in_desc]->datas['name'];
    if (!$name)
        return;

    begin_acls_use();
    echo "<h3>"._("This network object is routed on network:")."</h3>";
    print "<ul class='aclsuse'>";
    print "<li>".$name."</li>";
    print "</ul>\n";
    end_acls_use();
}

function checkAllAcls($expolicy, $object, $attr_names, $check_local_acl=true)
{
    global $elt, $used, $used_link;

    $used=array();
    $used['acls']=array();
    $used['localsin']=array();
    $used['localsout']=array();
    if ($check_local_acl) {
        $acls_array = Array('acls', 'localsin', 'localsout');
    } else {
        $acls_array = Array('acls');
    }
    foreach ($acls_array as $acltype)
    {
      $acls = $expolicy->$acltype;
      $all_acls= $acls->list_tab();
      if (!isset($all_acls))
      {
         $all_acls=array();
      }
      if (!is_array($attr_names))
      {
          $attr_names = Array($attr_names);
      }
      foreach ($all_acls as $acl_id=>$aclname)
      {
          $tmp_acl=$acls->get_elt($acl_id);
          foreach ($attr_names as $attr_name)
          {
            $value = $tmp_acl->$attr_name;
            if ((isset($value)) and $value!='')
                array_increment($used[$acltype], $value);
          }
      }
    }
    $used_link=array();
    $all_elts=$object->list_tab();
    if (!isset($all_elts))
    {
        $all_elts=array();
    }
    foreach ($all_elts as $id=>$eltname)
    {
        $l_elt=$object->get_elt($id);
	if (isset($l_elt))
          $all_sub_elts=$l_elt->list_tab();
        if (!isset($all_sub_elts))
        {
            $all_sub_elts=array();
        }
        foreach ($all_sub_elts as $sub_id=>$sub_eltname)
        {
            $tmp_sub_elt=$l_elt->get_elt($sub_id);
            if (($tmp_sub_elt->datas['type'] == 'link') and isset($tmp_sub_elt->datas['linkid']))
            {
                array_increment($used_link, $tmp_sub_elt->datas['linkid']);
            }
        }
        if ($elt==0 and count($all_sub_elts)==0)
        {
            $elt=$id;
            add_log(_("This object is shown by default because NuFace believes it is not complete"));
        }
    }
}

?>
